#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2021/12/8 14:57
@Author: lxc
@LastEditTime: 
@Desctiption: 聚鱼灯的连接器，控制聚鱼灯的开关
"""

import json
import threading
from connector import Connector
from event_storage import EventStorage
from log import OutPutLog
import time
import datetime
from modbus_tk import modbus_rtu_over_tcp


class GatherFishLightConnector(Connector, threading.Thread):

    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self._master = None
        self.__stopped = False
        self._connected = False
        self._ip = config['ip']  # ip
        self._port = config['port']  # 端口
        self._converter = converter
        self._storager = EventStorage()
        self._name = name
        self.run_time = 0
        self._command = json.loads(self._storager.get_command_info(name[:-3])[0]["command"])

    def open(self):
        self.__stopped = False
        self.start()


    def run(self):
        self._connect()
        self._connected = True

        while True:
            this_time = time.time()
            if this_time - self.run_time > 5:
                self.run_time = this_time

                light_info = self._storager.gather_fish_light_info(self._name[-2:])
                for each_light in light_info:
                    self.command_polling(each_light)


    def _connect(self):
        try:
            self._master = modbus_rtu_over_tcp.RtuOverTcpMaster(host=self._ip, port=self._port)
            print(f"{self._ip}:{self._port} connect success!")
        except Exception as e:
            print(f"{self._ip}:{self._port} connect failed!")
            self._log.error(f"{self._ip}:{self._port} connect failed! {e}")
            self._connected = False
            self._reconnect()


    def _reconnect(self):
        while True:
            try:
                self._master = modbus_rtu_over_tcp.RtuOverTcpMaster(host=self._ip, port=self._port)
                print(f"{self._ip}:{self._port} reconnect success!")
                print('client start connect to host/port:{}'.format(self._port))
                break
            except ConnectionRefusedError:
                print(f"{self._ip}:{self._port} modbus server refused or not started, reconnect to server in 5s .... ")
                time.sleep(5)
            except Exception as e:
                print(f"{self._ip}:{self._port} reconnect failed!")
                self._log.error(f"{self._ip}:{self._port} reconnect failed! {e}")
                time.sleep(5)


    def close(self):
        pass

    def get_name(self):
        return self.name

    def is_connected(self):
        return self._connected

    def send_command(self, data):

        result = True
        try:
            for each_command in data:
                result = self.exec_command(each_command)
        except Exception as e:
            print(f"[GatherFishLightConnector][send_command] error: {e}")
            result = False
        print("------[FishLight send_command]", self._ip, data, result)
        return result


    def exec_command(self, command):
        """
        0x01: 读线圈寄存器
        0x02: 读离散输入寄存器
        0x03: 读保持寄存器
        0x04: 读输入寄存器
        0x05: 写单个线圈寄存器
        0x06: 写单个保持寄存器
        0x0f: 写多个线圈寄存器
        0x10: 写多个保持寄存器
        #supported modbus functions
        READ_COILS = 1
        READ_DISCRETE_INPUTS = 2
        READ_HOLDING_REGISTERS = 3
        READ_INPUT_REGISTERS = 4
        WRITE_SINGLE_COIL = 5
        WRITE_SINGLE_REGISTER = 6
        READ_EXCEPTION_STATUS = 7
        DIAGNOSTIC = 8
        REPORT_SLAVE_ID = 17
        WRITE_MULTIPLE_COILS = 15
        WRITE_MULTIPLE_REGISTERS = 16
        READ_WRITE_MULTIPLE_REGISTERS = 23
        DEVICE_INFO = 43
        """
        if isinstance(command, str):
            command = json.loads(command)
        device_id = int(command['device_id'])
        function_code = int(command['function_code'])
        start_addr = int(command['start_addr'])
        output_value = command['output_value']

        if function_code in (1, 2, 3, 4):
            # 读寄存器
            pass
        elif function_code in (5, 6, 15, 16):
            # 写寄存器
            try:
                self._master.set_timeout(10.0)
                self._master.set_verbose(True)
                # print("device_id = ", device_id,  function_code, start_addr, output_value)
                data = self._master.execute(device_id, function_code, start_addr, output_value=output_value)
                res = command["res"]
                result = False
                if function_code == 5:
                    if start_addr == data[0] and res == data[1]:
                        result = True
                time.sleep(1)
                return result
            except Exception as e:
                self._log.error(e)
                return False
        else:
            self._log.debug(f'Unsupported function code.')

        # """
        # {
        #     "light_0": [
        #         {"res": 65280, "device_id": 1, "start_addr": 0, "output_value": 255, "function_code": 5},
        #         {"res": 65280, "device_id": 1, "start_addr": 1, "output_value": 255, "function_code": 5}
        #     ],
        #     "light_1": [
        #         {"res": 0, "device_id": 1, "start_addr": 0, "output_value": 0, "function_code": 5},
        #         {"res": 0, "device_id": 1, "start_addr": 1, "output_value": 0, "function_code": 5}
        #     ],
        #     "photosense_0": [
        #         {"res": 0, "device_id": 1, "start_addr": 2, "output_value": 255, "function_code": 5}
        #     ],
        #     "photosense_1": [
        #         {"res": 65280, "device_id": 1, "start_addr": 2, "output_value": 0, "function_code": 5}
        #     ]
        # }
        # """


    def command_polling(self, data, resend_times=None):
        '''判断当前模式需要发送的指令'''
        # data = {'control_mode': 1, 'status': 1, 'definite_time': [["10:00:00,11:00:00"], ["14:00:00,18:00:00"]]}
        control_mode = data["control_mode"]     # 0：光敏控制，1：手动控制，2：定时控制

        if control_mode == 2:
            '''定时模式'''
            light_status = 0
            now_time = datetime.datetime.now().strftime("%H:%M:%S")
            definite_time = json.loads(data["definite_time"])
            # 依次判断每个时间段
            for each_time in definite_time:
                each_time = each_time[0].split(",")
                light_open_time = each_time[0]
                light_close_time = each_time[1]
                if light_open_time < light_close_time:
                    if light_open_time <= now_time <= light_close_time:
                        light_status = 1
                else:
                    if now_time >= light_open_time or now_time <= light_close_time:
                        light_status = 1
            light_command = self._command[f"light_{light_status}"]
            command_res = True
            for each_command in light_command:
                res = self.exec_command(each_command)
            print("------", self._ip, light_command, "return=",res)



    # 原版：三种模式逻辑正常判断
    """
    def command_polling(self, data, resend_times=None):
        '''判断当前模式需要发送的指令'''
        # data = {'control_mode': 1, 'status': 1, 'definite_time': [["10:00:00,11:00:00"], ["14:00:00,18:00:00"]]}
        control_mode = data["control_mode"]     # 0：光敏控制，1：手动控制，2：定时控制
        photosense_status = 0                   # 光敏模式的状态，默认为关闭
        light_command = None

        if control_mode == 0:
            '''光敏模式'''
            photosense_status = 1
        else:
            light_status = 0        # 手动/定时模式时，默认聚鱼灯是关闭状态
            if control_mode == 1:
                '''手动模式'''
                light_status = data["status"]
            elif control_mode == 2:
                '''定时模式'''
                now_time = datetime.datetime.now().strftime("%H:%M:%S")
                definite_time = json.loads(data["definite_time"])
                # 依次判断每个时间段
                for each_time in definite_time:
                    each_time = each_time[0].split(",")
                    light_open_time = each_time[0]
                    light_close_time = each_time[1]
                    if light_open_time < light_close_time:
                        if light_open_time <= now_time <= light_close_time:
                            light_status = 1
                    else:
                        if now_time >= light_open_time or now_time <= light_close_time:
                            light_status = 1
            light_command = self._command[f"light_{light_status}"]

        # 开启/关闭光敏模式的指令
        command = self._command[f"photosense_{photosense_status}"]
        self.exec_command(command)
        # 非光敏模式下，控制聚鱼灯开关
        if light_command is not None:
            self.exec_command(light_command)

    # 特殊版：应对经海四号的光敏模式闪烁问题   就固定早7点关闭，晚6点开启
    def command_polling(self, data, resend_times=None):
        '''判断当前模式需要发送的指令'''
        # data = {'control_mode': 1, 'status': 1, 'definite_time': [["10:00:00,11:00:00"], ["14:00:00,18:00:00"]]}
        control_mode = data["control_mode"]     # 0：光敏控制，1：手动控制，2：定时控制
        photosense_status = 0                   # 光敏模式的状态，默认为关闭
        light_command = None

        if control_mode == 0:
            '''光敏模式'''
            # photosense_status = 1
            photosense_status = 0
            now = datetime.datetime.now().strftime("%H")
            if 7 <= int(now) < 18:
                # 关灯
                light_command = self._command["light_0"]
            else:
                # 开灯
                light_command = self._command["light_1"]
        else:
            light_status = 0        # 手动/定时模式时，默认聚鱼灯是关闭状态
            if control_mode == 1:
                '''手动模式'''
                light_status = data["status"]
            elif control_mode == 2:
                '''定时模式'''
                now_time = datetime.datetime.now().strftime("%H:%M:%S")
                definite_time = json.loads(data["definite_time"])
                # 依次判断每个时间段
                for each_time in definite_time:
                    each_time = each_time[0].split(",")
                    light_open_time = each_time[0]
                    light_close_time = each_time[1]
                    if light_open_time < light_close_time:
                        if light_open_time <= now_time <= light_close_time:
                            light_status = 1
                    else:
                        if now_time >= light_open_time or now_time <= light_close_time:
                            light_status = 1
            light_command = self._command[f"light_{light_status}"]
            # print("-------------------------------------------", self._ip, light_status, light_command)
        # 开启/关闭光敏模式的指令
        # command = self._command[f"photosense_{photosense_status}"]
        # self.exec_command(command)
        # 非光敏模式下，控制聚鱼灯开关
        if light_command is not None:
            self.exec_command(light_command)

    """


"""
光敏模式(开启) 发送：01 05 00 02 FF 00 2D FA   接收：01 05 00 02 FF 00 2D FA
光敏模式(关闭) 发送：01 05 00 02 00 00 6C 0A   接收：01 05 00 02 00 00 6C 0A
聚鱼灯开启：01 0F 00 00 00 04 01 01 01 00 00 5E 50
聚鱼灯关闭：01 0F 00 00 00 04 00 00 00 00 00 33 AC

01 ———— 设备地址
05 ———— 写单个线圈寄存器
00 02 ———— 起始寄存器地址
FF 00 ———— 寄存器的值
2D FA ———— CRC校验

01          ———— 设备地址
0F          ———— 功能码
00 00       ———— 起始地址
00 04       ———— 写入线圈个数
01          ———— 写入字节数
01 01 00 00 ———— 写入值
5E 50       ———— CRC校验

开启光敏模式 photosense_1= {"device_id":1, "function_code":5, "start_addr":2, "output_value":0}
关闭光敏模式 photosense_0={"device_id":1, "function_code":5, "start_addr":2, "output_value":65280}
开启聚鱼灯 light_1={"device_id":1, "function_code":15, "start_addr":0, "output_value":[1, 1, 0 ,0]}
关闭聚鱼灯 light_0={"device_id":1, "function_code":15, "start_addr":0, "output_value":[0, 0, 0 ,0]}
"""

"""
light_0：关灯
light_1：开灯
{
    "light_0": [
        {"res": 0, "device_id": 1, "start_addr": 0, "output_value": 0, "function_code": 5}, 
        {"res": 0, "device_id": 1, "start_addr": 1, "output_value": 0, "function_code": 5}
    ], 
    "light_1": [
        {"res": 65280, "device_id": 1, "start_addr": 0, "output_value": 65280, "function_code": 5}, 
        {"res": 65280, "device_id": 1, "start_addr": 1, "output_value": 65280, "function_code": 5}
    ]
}
开启光敏模式 photosense_1= {"device_id":1, "function_code":5, "start_addr":2, "output_value":0}
关闭光敏模式 photosense_0={"device_id":1, "function_code":5, "start_addr":2, "output_value":65280}
"""