import json
import queue
import time
import threading
import datetime
import ast
from connector import Connector
from event_storage import EventStorage
from libs.hls import SiemensS7Net
from libs.hls import SiemensPLCS
from log import OutPutLog

class SiemensPlcS7Connector(Connector, threading.Thread):
    __master = 0
    _disConnectTime = 0

    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self.__master = None
        self.__stopped = False
        self.__connected = False
        self._ip = config['ip']  # ip
        self._port = config['port']  # 端口
        self.__save_frequency = config['save_frequency']  # 数据存储时间间隔
        self.setDaemon(True)
        self.setName(name)
        self.__converter = converter
        self.__storager = EventStorage()
        self.__command_queue = queue.Queue(500)
        self.__last_save_time = 0
        self.__data_point_config = self.__storager.get_station_info(name)
        self.__command = self.__storager.get_command_info(name)


    def open(self):
        self.__stopped = False
        self.start()

    def run(self):
        self.__connect()
        command_list = json.loads(self.__command[0]['command'])
        while True:
            time.sleep(0.1)
            self.command_polling(command_list, resend_times=5)
            if self.__stopped:
                break

    def __connect(self):
        try:
            self.__master = SiemensS7Net(SiemensPLCS.S1200, self._ip, self._port)
            if self.__master.ConnectServer().IsSuccess == False:
                self.__connected = False
                #print("s7 connect falied")
            else:
                self.__connected = True
                #print("s7 connect success")
        except Exception as e:
            self._log.error(e)

    def __reconnect(self):
        pass

    def close(self):
        pass

    def get_name(self):
        return self.name

    def is_connected(self):
        return self.__connected

    def send_command(self, data):
        try:
            for command in data:
                self.__command_queue.put(command)
            return "success"
        except Exception as e:
            print(e)
            return "send_command error"+str(e)

    def command_polling(self, command_list, resend_times=None):
        #msg = str(time.strftime("%Y-%m-%d %H:%M:%S"))
        for i in range(len(command_list)):
            command_item = command_list[i]
            if not self.__command_queue.empty():
                write_command = self.__command_queue.get()  # 写命令来自数组
                try:
                    self.writeData(write_command['addr_data'],write_command['data_type'] , write_command['data'])
                    #self._log.info('success'+str(res))
                except Exception as e:
                    print("S7Converter,write[ERROR]:" + str(e))
                    self._log.error("S7Converter,write[ERROR]:" + str(e))
                '''
                if not res:
                    # 根据（轮询预计用时，剩余可用时间）计算重发次数
                    sent_times = 1
                    while sent_times < resend_times:
                        res = self.exec_command(write_command)
                        sent_times += 1
                        if res:
                            break
                '''
            else:
                # 格式化数据
                result = {}
                start_addr = command_item['start_addr']
                length = command_item['length']
                db_number = "".join(start_addr).split(".")[0]
                result[db_number] = self.read_batch_data(start_addr, length)
                # print("-----------------", self._ip, self._port, start_addr, length, result)
                format_data = self.__converter.convert(self.__data_point_config, result, self.__master)
                if format_data:
                    if format_data != "error" and format_data != 'pass':
                         # 往redis存储数据
                        self.__storager.real_time_data_storage(format_data)
                        self._log.info('success -- 实时数据存入redis！')

    def read_batch_data(self, addr, length):
        try:
            result = self.__master.Read(addr, length)
            if result.IsSuccess:
                datas = result.Content
                return datas
            else:
                if self.__master.ConnectServer().IsSuccess == False:
                    self.__connected = False
                    #print("s7 connect falied")
                else:
                    self.__connected = True
                    #print("s7 connect success")
                return None
        except Exception as e:
            print("S7Converter,read[ERROR]:" + str(e))
            return None

    def writeData(self, address, data_type, data):
        data_addr = self.address_transform(address)
        if data_type == "BOOL":
            data = int(data)
            self.__master.WriteBool(data_addr, data)
        elif data_type == "DTL":
            data_conversion = data.split(" ")
            data_conversion_date = data_conversion[0].split("-")
            data_conversion_time = data_conversion[1].split(":")
            year_h = int(data_conversion_date[0]) >> 8
            year_l = int(data_conversion_date[0]) - year_h * 256
            write_month = int(data_conversion_date[1])
            write_days = int(data_conversion_date[2])
            write_hours = int(data_conversion_time[0])
            write_min = int(data_conversion_time[1])
            write_sec = int(data_conversion_time[2])
            data_in = bytes([year_h, year_l, write_month, write_days, 0, write_hours, write_min, write_sec, 0, 0, 0, 0])
            self.__master.Write(data_addr, data_in)
        elif data_type == "FLOAT32":
            data = float(data)
            self.__master.WriteFloat(data_addr, data)
        elif data_type == "UINT8":
            data = int(data)
            self.__master.WriteByte(data_addr, data)

    def readData(self, addr_data, data_type):

        data_addr = self.address_transform(addr_data)
        if data_type == "BOOL":
            data_is = self.__master.ReadBool(data_addr).Content  # 读布尔类型
            return data_is
        elif data_type == "DTL":
            data_is = self.__master.Read(data_addr, 12).Content  # 读布尔类型ReadDouble
            year = int(data_is[0]) * 256 + int(data_is[1])
            month = int(data_is[2])
            day = int(data_is[3])
            hour = int(data_is[5])
            minute = int(data_is[6])
            sec = int(data_is[7])
            times = str(year) + "-" + str(month) + "-" + str(day) + " " + str(hour) + ":" + str(minute) + ":" + str(sec)
            timeStruct = time.strptime(times, "%Y-%m-%d %H:%M:%S")
            times = time.strftime("%Y-%m-%d %H:%M:%S", timeStruct)
            return times
        elif data_type == "FLOAT":
            data_is = self.__master.ReadFloat(data_addr).Content
        elif data_type == "UINT8":
            data_is = self.__master.ReadByte(data_addr).Content
        return data_is

    def address_transform(self, addr):
        real_addr_buffer = "".join(addr).split(".")  # 合并list为字符串 用“.”分割字符串获得数据
        if real_addr_buffer[1][2] == 'X':
            data_addr = real_addr_buffer[0] + "." + real_addr_buffer[1][3:] + "." + real_addr_buffer[2]
        elif real_addr_buffer[1][2] == 'D' or real_addr_buffer[1][2] == 'B':
            data_addr = real_addr_buffer[0] + "." + real_addr_buffer[1][3:]
        return data_addr
